# Autogenerated by devscripts/update-version.py

__version__ = '2026.02.21'

RELEASE_GIT_HEAD = '646bb31f39614e6c2f7ba687c53e7496394cbadb'

VARIANT = None

UPDATE_HINT = None

CHANNEL = 'stable'

ORIGIN = 'yt-dlp/yt-dlp'

_pkg_version = '2026.02.21'
